// This file is part of Dust Racing 2D.
// Copyright (C) 2012 Jussi Lind <jussi.lind@iki.fi>
//
// Dust Racing 2D is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// Dust Racing 2D is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Dust Racing 2D. If not, see <http://www.gnu.org/licenses/>.

#ifndef MENUITEM_HPP
#define MENUITEM_HPP

#include <functional>
#include <string>
#include <memory>

namespace MTFH {

class Menu;
class MenuItemAction;
class MenuItemView;

//! Base class for items used in menus.
class MenuItem
{
public:

    MenuItem(int width, int height, std::string text = "", bool selectable = false);

    virtual ~MenuItem();

    void setIndex(int index);

    int index() const;

    int width() const;

    int height() const;

    void setPos(int x, int y);

    int x() const;

    int y() const;

    const std::string & text() const;

    void setView(MenuItemView * view, bool takeOwnership = false);

    MenuItemView * view();

    void setAction(MenuItemAction * action, bool takeOwnership = false);

    void setAction(std::function<void()> actionFunction);

    void setMenuOpenAction(const std::string & menuId);

    MenuItemAction * action() const;

    virtual void render();

    virtual void onLeft();

    virtual void onRight();

    virtual void onUp();

    virtual void onDown();

    virtual void setSelected(bool flag);

    virtual void setFocused(bool focused);

    void setContentsMargins(int left, int right, int top, int bottom);

    void getContentsMargins(int & left, int & right, int & top, int & bottom) const;

    bool selected() const;

    bool selectable() const;

    bool focused() const;

private:

    std::string m_text;
    std::string m_menuOpenActionMenuId;
    MenuItemAction * m_action;
    std::function<void()> m_actionFunction;
    MenuItemView * m_view;
    typedef std::shared_ptr<MenuItemAction> MenuItemActionPtr;
    MenuItemActionPtr m_ownedAction;
    typedef std::shared_ptr<MenuItemView> MenuItemViewPtr;
    MenuItemViewPtr m_ownedView;
    bool m_focused;
    bool m_selected;
    bool m_selectable;
    int m_width, m_height, m_x, m_y;
    int m_lMargin, m_rMargin, m_tMargin, m_bMargin;
    int m_index;
};

} // namespace MTFH

#endif // MENUITEM_HPP
